#pragma option v;
/*
   These routines may be adapted for any purpose when
   used with the MPC Code Development system.  No
   warranty is implied or given as to their usability
   for any purpose.

        (c) Copyright 1995

        Byte Craft Limited
        Waterloo, Ontario
        Canada N2J 4E4
        (519) 888-6911

        Sherif Abdel-Kader


   This is a simple example on using the A/D
   converter on the PIC16C71.

   - init_ADC() sets up analog inputs used,
     configures corresonding pins as inputs,
     and sets the oscillator type.

   - Select_ADCch() selects the analog channel
     for the next conversion.

   - ADC_On() turns the A/D module on and waits
     for a sample delay period.

   - ConvertAD() starts the conversion, loops
     till conversion completed, then returns
     digital value.

 NOTES:
   System clock speed = 16Mhz
   Instuction clock = 4Mhz
   Cycle duration = 250 ns
   Sampling time required = 10us (refer to data book)
*/

#include <16C71.h>

#define SAMPLE_DELAY 10      // 10 us
#define _DONE         2      // Conversion done

typedef unsigned int BYTE;

void init_ADC(bits ADConf);
void Select_ADch(bits channel);
void ADC_On(void);
BYTE ConvertAD(void);

//////////////////////////////////////////////////
void main()
{
 BYTE result;

 init_ADC(0b00110001);  // RC osc., RA0
 Select_ADch(AIN0);     // Channel 0
 ADC_On();
 result = ConvertAD();
}

///////////////////////////////////////////////////
//   bits ADConf;
//   ------------
//    7   6    5     4    3   2   1   0
//
//    -   -  ADCS1 ADCS0 RA3 RA2 RA1 RA0
//---------------------------------------
//   Channels used        ADConf bits set
//     NONE                   NONE
//     RA0                     <0>
//     RA0,RA1               <1:0>
//     RA0-RA2 (RA3=VREF)    <2:0>
//     RA0-RA3               <3:0>
//
//   Conversion clock      ADConf<5:4>
//     fosc/2                  00
//     fosc/8                  01
//     fosc/32                 10
//     fRC                     11
//
void init_ADC(bits ADConf)
{
 // Pins must be in input mode
 // TRISA<3:0> = ADConf<3:0>
  TRISA = TRISA | (ADConf & 0x0F);

 // Select conversion clock
  ADCON0.ADCS0 = ADConf.4;
  ADCON0.ADCS1 = ADConf.5;

 // Set appropriate analog input configuration
  switch(ADConf)
   {
    case 0x01:   // RA0 only
    case 0x03:   // RA0 and RA1
          ADCON1 = 0x02;
         break;
    case 0x07:   // RA0-RA2, RA3=VREF
          ADCON1 = 0x01;
         break;
    case 0x0F:   // RA0-RA3
          ADCON1 = 0x00;
         break;
   } // switch
}

//////////////////////////////////////////////////
void Select_ADch(bits channel)
{
 ADCON0.CHS0 = channel.0;
 ADCON0.CHS1 = channel.1;
}

////////////////////////////////////////////////
void delay(BYTE duration) // duration is in us
{
 #asm
DL
   NOP
   DECFSZ  duration
   goto DL
 #endasm
}

///////////////////////////////////////////////
void ADC_On()
{
 ADCON0.ADON = 1;      // ADC module on
 delay(SAMPLE_DELAY);  // Sample delay
}

//////////////////////////////////////////////
BYTE ConvertAD(void)
{
 ADCON0.2 = 1;         // Start conversion
 while(ADCON0._DONE);  // Loop till done.
 return(ADRES);
}

//////////////////////////////////////////////

