#pragma option v
/*
   These routines may be adapted for any purpose when
   used with the MPC Code Development system.  No
   warranty is implied or given as to their usability
   for any purpose.

        (c) Copyright 1995

        Byte Craft Limited
        Waterloo, Ontario
        Canada N2J 4E4
        (519) 888-6911

        Sherif Abdel-Kader
*/
// This example demonstrates writing interrupt
// service routines for the PIC16C64.
//
// When an interrupt occurs, the PC jumps to
// location 0004 in the program memory. In the
// 16C64 header file, an interrupt vector
// called __INT is defined at that location.
// If we create a function with the same name
// it is called upon occurrence of the interrupt.
// You can change the function name by changing
// the vector name in the 16C64 header file.
// Inside the function we determine the source
// of the interrupt and call an appropriate
// handler.
// There are eight sources of interrupt in the
// PIC16C64. In this program we only use three.
// In the event of an unexpected interrupt
// __INT function will be called repetitively.
// In this case pin 0 on PORTC will be toggled
// to indicate such condition on an emulator for
// example.
// Note the saving and restoring of the W and
// STATUS registers when servicing an interrupt.
// The RP0 flag in particular is essentially
// important since the interrupt service routine
// may be called while referencing a variable
// and hence reference an incorrect register file.
// Also the Z flag can be altered while saving
// the STATUS register so it is saved too.
// __WImage is declared internally by the compiler
// only if it needs a scratch pad location for
// some operations. It is safer, however to
// declare __WImage and save it during interrupts.

#include <16C64.h>

void handle_PORTB();
void handle_EXTERN();
void handle_TMR0();

bits temp_STATUS;     // temporary storage for STATUS
int  temp_WREG;       // temporary storage for W
int  __WImage;        // Used by the compiler
int  temp_WImage;     // temporary storage for __WImage

///////////////////////////////////////////////////////////

void __INT(void)
{
// Save W ad STATUS regiseters
  if (STATUS.RP0) {  // RP0 = 1
    if (STATUS.Z) {  // Z = 1
      temp_WREG = WREG;
      temp_STATUS = STATUS;
      temp_STATUS.Z = 1;
    } // if
    else {           // Z = 0
      temp_WREG = WREG;
      temp_STATUS = STATUS;
      temp_STATUS.Z = 0;
    } // else
    temp_STATUS.RP0 = 1;
  } // if
  else {             // RP0 = 0
    if (STATUS.Z) {  // Z = 1
      temp_WREG = WREG;
      temp_STATUS = STATUS;
      temp_STATUS.Z = 1;
    } // if
    else {           // Z = 0
      temp_WREG = WREG;
      temp_STATUS = STATUS;
      temp_STATUS.Z = 0;
    } // else
    temp_STATUS.RP0 = 0;
  } // else
  temp_WImage = __WImage;

// Check which interrupt occurred
  if (INTCON.RBIF) { // Change on RB<7:4>
    INTCON.RBIF = 0; // Clear RBIF
    handle_PORTB();  // Call handler
  } // if
  if (INTCON.INTF) { // External int.
    INTCON.INTF = 0; // Clear INTF
    handle_EXTERN(); // Call handler
  } // if
  if (INTCON.T0IF) { // TMR0 overflowed
    INTCON.T0IF = 0; // Clear T0IF
    handle_TMR0();   // Call handler
  } // if
  else  {  // Unexpected interrupt occurred.
    TRISC &= 0xFE;     // Use pin 0 on...
      for (;;) {
        PORTC.0 = 0;     // ...PORTC to indicate
        PORTC.0 = 1;     // unexpected interrupt
      }
  } // else


  /* Restore registers and flags */

  __WImage = temp_WImage;        // Restore __WImage
  STATUS = temp_STATUS;          // Restore STATUS
  WREG = temp_WREG;              // ..then WREG
  STATUS.RP0 = temp_STATUS.RP0;  // ..then RP0
  STATUS.Z = temp_STATUS.Z;      // ..then Z
} // __INT

///////////////////////////////////////////////////////////

void main()
{
  // Disable and clear all interrupts
     INTCON = 0;
     PIR1 = 0;
     PIE1 = 0;
  // Enable desired interrupts
     INTCON.RBIE = 1;      // Change on PORTB
     INTCON.INTE = 1;      // External interrupt
     INTCON.T0IE = 1;      // Timer 0 overflow interrupt
  // Enable unmasked interrupts
     INTCON.GIE = 1;       // Global interrupt enable
  // Loop until an interrupt occurs
     while(1) {
     }
} // main

///////////////////////////////////////////////////////////

void handle_PORTB()
{
  // Code for handling PORTB change interrupt
  //
}

///////////////////////////////////////////////////////////

void handle_EXTERN()
{
  // Code for handling external interrupt
  //
}

///////////////////////////////////////////////////////////

void handle_TMR0()
{
  // Code for handling TMR0 overflow interrupt
  //
}
//////////////////////////////////////////////////////////

